#!/usr/bin/env python

import trust
import trust.audit
import logging
import os
import www
from flask import Flask, jsonify, request
from logging.handlers import SysLogHandler


app = Flask(__name__)

logHandler = SysLogHandler(address="/dev/log")
formatter = logging.Formatter("trust-service: [%(levelname)s] %(message)s")

logHandler.setFormatter(formatter)
logHandler.setLevel(logging.DEBUG)
app.logger.addHandler(logHandler)
app.logger.setLevel(logging.DEBUG)

trust.trust_blueprint.finder = trust.FileFinder(
    data_path=os.path.join(os.path.dirname(__file__), "tests/system/data"),
    logger=app.logger,
    audit=trust.audit.AuditToSyslog("trust-service-audit"))

app.register_blueprint(trust.trust_blueprint, url_prefix="/service")
app.register_blueprint(www.docs_blueprint)

app.logger.info("The application started.")

if __name__ == "__main__":
    app.run(threaded=True)



import trust
import trust.formatters


class Trust():
    def __init__(self, finder):
        self._finder = finder

    def process(self, query, response_mode, credentials, optional=False):
        formatter = self._map_formatter(response_mode)
        if not self._is_query_valid(query):
            return formatter.reject_invalid()

        return formatter.process(query, credentials, optional)

    def _is_query_valid(self, query):
        if "/../" in query or query.endswith("/.."):
            return False

        if query.startswith("/_users/") or query == "/_users":
            return False

        if query.startswith("/_groups/") or query == "/_groups":
            return False

        return True

    def _map_formatter(self, response_mode):
        formatter_type = {
            "json": trust.formatters.JsonFormatter,
            "text": trust.formatters.TextFormatter,
            "complete": trust.formatters.CompleteFormatter,
        }.get(response_mode or "complete", None)

        if formatter_type is None:
            raise trust.FormatterException(
                ("The formatter corresponding to response mode %s doesn't " +
                 "exist.") % (response_mode,)
            )

        return formatter_type(self._finder)


class Credentials():
    def __init__(self, username, password):
        self._username = username
        self._password = password

    @property
    def username(self):
        return self._username

    @property
    def password(self):
        return self._password

    @property
    def are_empty(self):
        return self._username is None

    @staticmethod
    def get_empty():
        return Credentials(None, None)



import time
from passlib.hash import pbkdf2_sha256


class InDataAuthProvider():
    def __init__(self, finder, logger):
        self._finder = finder
        self._logger = logger
        self._logger.debug("In-data authentication provider initialized.")

    def verify(self, credentials):
        self._logger.debug("Authentication provider is verifying a user.")
        password_hash = self._finder.find(
            "/_users/" + credentials.username + "/hash"
        )

        start_time = time.time()
        result = pbkdf2_sha256.verify(credentials.password, password_hash)
        end_time = time.time()
        self._logger.debug(
            "PBKDF2 verification done in %s ms. resulting in %s.",
            int(end_time - start_time),
            result)

        return result

    def ingroup(self, username, groupname):
        return groupname in self._get_groups(
            "/_users/" + username + "/member-of"
        )

    def _get_groups(self, path):
        """
        Retrieves the groups the user or group belongs to.
        """
        groups = list(set(self._finder.find(path, optional=True) or []))

        i = 0
        while i < len(groups):
            current = groups[i]
            i = i + 1
            yield current

            child_path = "/_groups/" + current + "/member-of"
            children = self._finder.find(child_path, optional=True) or []
            self._add_unique(groups, children)

    @staticmethod
    def _add_unique(current_list, values):
        """
        Extends a list by adding the values which are not yet in the list.
        """
        current_list.extend([c for c in values if c not in current_list])



#!/usr/bin/env python

import argparse
import logging
import trust
from logging.handlers import SysLogHandler


def parseArgs():
    description = ("Extracts data from a JSON file, multiple files or MongoDB "
                   "database.")

    parser = argparse.ArgumentParser(description=description)

    parser.add_argument(
        "source",
        help="The location of the source data."
    )

    parser.add_argument(
        "--response-mode",
        choices=["json", "text", "complete"],
        default="complete",
        help="The way to format the response."
    )

    parser.add_argument(
        "--username",
        help="The name of the user accessing the data."
    )

    parser.add_argument(
        "--password",
        help="The password of the user accessing the data."
    )

    parser.add_argument(
        "--optional",
        action='store_true',
        help="The value is optional and may not exist in the source data."
    )

    parser.add_argument(
        "query",
        help="The query path."
    )

    return parser.parse_args()


if __name__ == "__main__":
    logger = logging.getLogger("trust")
    logHandler = SysLogHandler(address="/dev/log")
    formatter = logging.Formatter("trust: [%(levelname)s] %(message)s")
    logHandler.setFormatter(formatter)
    logHandler.setLevel(logging.DEBUG)
    logger.addHandler(logHandler)
    logger.setLevel(logging.DEBUG)

    args = parseArgs()

    finder = trust.FileFinder(
        data_path=args.source,
        logger=logger,
        audit=trust.audit.AuditToSyslog("trust-audit")
    )

    engine = trust.Trust(finder)
    response = engine.process(
        args.query,
        args.response_mode,
        trust.Credentials(args.username, args.password),
        args.optional,
    )

    print(response)



import json
import trust
import trust.formatters
from trust.exceptions import AuthenticationException
from .formatter_tests import FormatterTests, FinderStubWithValue


class TestJsonFormatter(FormatterTests):
    def test_none(self):
        actual = self._process_default(None)
        expected = '{"result": null}'
        self.assertEqual(expected, actual)

    def test_int(self):
        actual = self._process_default(5)
        expected = '{"result": 5}'
        self.assertEqual(expected, actual)

    def test_float(self):
        actual = self._process_default(5.0)
        expected = '{"result": 5.0}'
        self.assertEqual(expected, actual)

    def test_string(self):
        actual = self._process_default("Hello")
        expected = '{"result": "Hello"}'
        self.assertEqual(expected, actual)

    def test_object(self):
        actual = self._process_default({"Hello": 2})
        expected = '{"result": {"Hello": 2}}'
        self.assertEqual(expected, actual)

    def test_list(self):
        actual = self._process_default([3, 7, 1])
        expected = '{"result": [3, 7, 1]}'
        self.assertEqual(expected, actual)

    def test_list_of_objects(self):
        actual = self._process_default([3, "abc", {"hello": "world"}])
        expected = '{"result": [3, "abc", {"hello": "world"}]}'
        self.assertEqual(expected, actual)

    def test_wrong_credentials(self):
        class FinderStub():
            def access_as(self, credentials):
                pass

            def find(self, query, optional):
                raise AuthenticationException()

        result = self._get_formatter(FinderStub()).process(
            "/", trust.Credentials("Lucy", "demo")
        )

        self._logger.debug("Got response %s.", result)
        actual = json.loads(result)["errors"][0]["type"]
        expected = "credentials-invalid"
        self.assertEqual(expected, actual)

    def test_no_enough_permissions(self):
        class FinderStub():
            def access_as(self, credentials):
                pass

            def find(self, query, optional):
                raise PermissionError()

        result = self._get_formatter(FinderStub()).process(
            "/", trust.Credentials("Lucy", "demo")
        )

        self._logger.debug("Got response %s.", result)
        actual = json.loads(result)["errors"][0]["type"]
        expected = "permission-required"
        self.assertEqual(expected, actual)

    def _get_formatter(self, finder):
        return trust.formatters.JsonFormatter(finder)



class Zipper():
    def __init__(self, child, parent, logger=None):
        self._child = child
        self._parent = parent
        self._logger = logger or logging.getLogger(__name__)

    def zip(self):
        actions = ["merge"]

        if type(self._child) is not dict:
            self._logger.debug("Child is not a dict. Returning.")
            return self._child
        else:
            self._logger.debug("Child as keys %s.", self._child.keys())
            if ".special:actions" in self._child.keys():
                actions = self._child[".special:actions"]
                self._logger.debug("Using custom actions %s.", actions)
                if type(self._parent) is list:
                    return self._combine_with_parent_list(actions)

        if type(self._parent) is not dict:
            return self._parent

        return self._combine_element(
            actions,
            self._combine_on_both,
            self._combine_on_child,
            self._combine_on_parent
        )

    def _combine_with_parent_list(self, actions):
        """
        Combines a child and a parent when the parent is a list.
        """
        if "add" in actions:
            return self._add_array()

        if "merge" in actions:
            return self._merge_array()

    def _add_array(self):
        values_to_add = self._child[".special:values"]
        return self._parent + values_to_add

    def _merge_array(self):
        return list(set(self._add_array()))

    def _combine_on_both(self, actions, child_element, parent_element):
        if "replace" in actions or "merge" in actions:
            return Zipper(child_element, parent_element, self._logger).zip()

    def _combine_on_child(self, actions, child_element):
        if "replace" in actions or "merge" in actions:
            return child_element

    def _combine_on_parent(self, actions, parent_element):
        if "replace" in actions:
            # Do nothing, since child is replacing the parent.
            return None

        if "merge" in actions:
            return parent_element

    def _combine_element(self, actions, on_both, on_child, on_parent):
        child_keys = (self._child.keys()
                      if type(self._child) is dict
                      else [])

        parent_keys = (self._parent.keys()
                       if type(self._parent) is dict
                       else [])

        parent_items = (self._parent.items()
                        if type(self._parent) is dict
                        else [])

        result = {}

        for parent_key, parent_value in parent_items:
            if parent_key in child_keys:
                # The element exists both in child and parent.
                self._logger.info(
                    "Combining parent and child %s using actions %s.",
                    parent_key, actions
                )

                new = on_both(actions, self._child[parent_key], parent_value)
                if new:
                    result[parent_key] = new

            elif parent_key != ".special:actions":
                # The element exists only in parent.
                self._logger.info(
                    "Combining parent %s using actions %s.",
                    parent_key, actions
                )

                new = on_parent(actions, parent_value)
                if new:
                    result[parent_key] = new

        for child_key, child_value in self._child.items():
            if child_key != ".special:actions":
                if child_key not in parent_keys:
                    # The element exists only in child.
                    self._logger.info(
                        "Combining child %s using actions %s.",
                        child_key, actions
                    )

                    new = on_child(actions, child_value)
                    if new:
                        result[child_key] = new

        return result



import logging
import unittest


class LoggedTests(unittest.TestCase):
    def setUp(self):
        logging.basicConfig(level=logging.DEBUG)
        self._logger = logging.getLogger(__name__)
        print()
        self._logger.info(
            "--- Testing %s.%s ---",
            self.__class__.__name__,
            self._testMethodName)



import trust
import trust.formatters
from trust.exceptions import AuthenticationException
from .formatter_tests import FormatterTests, FinderStubWithValue


class TestTextFormatter(FormatterTests):
    def test_none(self):
        actual = self._process_default(None)
        expected = ""
        self.assertEqual(expected, actual)

    def test_int(self):
        actual = self._process_default(5)
        expected = "5"
        self.assertEqual(expected, actual)

    def test_float(self):
        actual = self._process_default(5.0)
        expected = "5.0"
        self.assertEqual(expected, actual)

    def test_string(self):
        actual = self._process_default("Hello")
        expected = "Hello"
        self.assertEqual(expected, actual)

    def test_object(self):
        actual = self._process_default({"Hello": 2})
        expected = '{"Hello": 2}'
        self.assertEqual(expected, actual)

    def test_list(self):
        actual = self._process_default([3, 7, 1])
        expected = "3\n7\n1"
        self.assertEqual(expected, actual)

    def test_list_of_objects(self):
        actual = self._process_default([3, "abc", {"hello": "world"}])
        expected = '3\nabc\n{"hello": "world"}'
        self.assertEqual(expected, actual)

    def test_wrong_credentials(self):
        class FinderStub():
            def access_as(self, credentials):
                pass

            def find(self, query, optional):
                raise AuthenticationException

        with self.assertRaises(AuthenticationException) as context:
            self._get_formatter(FinderStub()).process(
                "/", trust.Credentials("Lucy", "demo")
            )

    def _get_formatter(self, finder):
        return trust.formatters.TextFormatter(finder)



import trust
import flask
import os


# In order to use this blueprint, the following property should be set:
#
#    finder: contains an instance of the finder which will actually access the
#            data.

trust_blueprint = flask.Blueprint("trust", __name__)


@trust_blueprint.route("/", defaults={"path": ""}, methods=["GET"])
@trust_blueprint.route("/<path:path>", methods=["GET"])
def handle_all(path):
    try:
        if path == "favicon.ico":
            flask.current_app.logger.info(
                "favicon.ico was requested to Trust blueprint which will "
                "respond with HTTP 404.")
            return "", 404

        request = flask.request
        query = "/" + path
        response_mode = request.args.get("response-mode", "json")
        optional = request.args.get("optional", 0) == 1
        credentials = (
            request.authorization
            if request.authorization and request.authorization.username
            else trust.Credentials.get_empty()
        )

        engine = trust.Trust(trust_blueprint.finder)
        response = engine.process(query, response_mode, credentials, optional)
        headers = {
            "Content-Length": str(len(response)),
            "Content-Type": _get_mime(response_mode) + "; charset=utf-8"
        }

        return response, 200, headers

    except Exception as e:
        flask.current_app.logger.exception(e)
        raise


def _get_mime(response_mode):
    return {
        "json": "application/json"
    }.get(response_mode, "text/plain")



#!/usr/bin/env python

import trust
import trust.audit
import logging
import os
from flask import Flask, jsonify, request
from logging.handlers import SysLogHandler


flaskApp = Flask(__name__)

logHandler = SysLogHandler(address="/dev/log")
formatter = logging.Formatter("trust-service: [%(levelname)s] %(message)s")

logHandler.setFormatter(formatter)
logHandler.setLevel(logging.INFO)
flaskApp.logger.addHandler(logHandler)
flaskApp.logger.setLevel(logging.INFO)

trust.trust_blueprint.finder = trust.FileFinder(
    data_path=os.path.join(os.path.dirname(__file__), "tests/system/data"),
    logger=flaskApp.logger,
    audit=trust.audit.AuditToSyslog("trust-service-audit"))

flaskApp.register_blueprint(trust.trust_blueprint)

flaskApp.logger.info("The application started.")

if __name__ == "__main__":
    flaskApp.run()



import trust
import trust.exceptions
from .common_tests import CommonTests


class TestFileFinderSecurity(CommonTests):
    def test_guest(self):
        self._writeJson(
            "hello.json", {
                ".special:restricted": {}
            }
        )

        with self.assertRaises(PermissionError) as context:
            self._finder.find("/hello")

    def test_guest_child(self):
        self._writeJson(
            "hello.json", {
                ".special:restricted": {},
                "world": "Hello"
            }
        )

        with self.assertRaises(PermissionError) as context:
            self._finder.find("/hello/world")

    def test_guest_parent(self):
        self._writeJson(
            "hello.json", {
                "hello": "Hello, World!",
                "world": {
                    ".special:restricted": {},
                }
            }
        )

        actual = self._finder.find("/hello")
        expected = {
            "hello": "Hello, World!",
            "world": None
        }

        self.assertEqual(expected, actual)

    def test_guest_inheritance(self):
        self._writeJson(
            "parent.json", {
                ".special:restricted": {}
            }
        )

        self._writeJson(
            "child.json", {
                ".special:inherit": "/parent",
                "hello": 2
            }
        )

        with self.assertRaises(PermissionError) as context:
            self._finder.find("/child/world")

    def test_user_getter_default(self):
        actual = self._finder.username
        self.assertIsNone(actual)

    def test_user_getter(self):
        class AuthStub():
            def verify(self, credentials):
                return True

        finder = self._finder
        finder.auth_provider = AuthStub()
        finder.access_as(trust.Credentials("John", "demo"))
        actual = finder.username
        expected = "John"
        self.assertEqual(expected, actual)

    def test_authentication_invalid(self):
        class AuthStub():
            def verify(self, credentials):
                return False

        self._writeJson(
            "hello.json", {
                ".special:restricted": {
                    "users": ["John"]
                },
                "world": "Hello"
            }
        )

        finder = self._finder
        finder.auth_provider = AuthStub()
        finder.access_as(trust.Credentials("John", "demo"))
        with self.assertRaises(
                trust.exceptions.AuthenticationException) as context:
            finder.find("/hello")

    def test_user(self):
        class AuthStub():
            def verify(self, credentials):
                return True

        self._writeJson(
            "hello.json", {
                ".special:restricted": {
                    "users": ["John"]
                },
                "world": "Hello"
            }
        )

        finder = self._finder
        finder.auth_provider = AuthStub()
        finder.access_as(trust.Credentials("John", "demo"))
        actual = finder.find("/hello")
        expected = {
            "world": "Hello"
        }

        self.assertEqual(expected, actual)

    def test_user_child(self):
        class AuthStub():
            def verify(self, credentials):
                return True

        self._writeJson(
            "hello.json", {
                ".special:restricted": {
                    "users": ["John"]
                },
                "world": "Hello"
            }
        )

        finder = self._finder
        finder.auth_provider = AuthStub()
        finder.access_as(trust.Credentials("John", "demo"))
        actual = finder.find("/hello/world")
        expected = "Hello"
        self.assertEqual(expected, actual)

    def test_user_parent(self):
        class AuthStub():
            def verify(self, credentials):
                return True

        self._writeJson(
            "hello.json", {
                "child": {
                    ".special:restricted": {
                        "users": ["John"]
                    },
                    "world": "Hello"
                }
            }
        )

        finder = self._finder
        finder.auth_provider = AuthStub()
        finder.access_as(trust.Credentials("John", "demo"))
        actual = finder.find("/hello")
        expected = {
            "child": {
                "world": "Hello"
            }
        }

        self.assertEqual(expected, actual)

    def test_user_outside_list(self):
        class AuthStub():
            def verify(self, credentials):
                return True

        self._writeJson(
            "hello.json", {
                ".special:restricted": {
                    "users": ["Lucy"]
                },
                "world": "Hello"
            }
        )

        finder = self._finder
        finder.auth_provider = AuthStub()
        finder.access_as(trust.Credentials("John", "demo"))
        with self.assertRaises(PermissionError) as context:
            actual = finder.find("/hello/world")

    def test_group(self):
        class AuthStub():
            def verify(self, credentials):
                return True

            def ingroup(self, username, groupname):
                return True

        self._writeJson(
            "hello.json", {
                ".special:restricted": {
                    "users": ["Lucy"],
                    "groups": ["Backup operators"]
                },
                "world": "Hello"
            }
        )

        finder = self._finder
        finder.auth_provider = AuthStub()
        finder.access_as(trust.Credentials("John", "demo"))
        actual = finder.find("/hello/world")
        expected = "Hello"
        self.assertEqual(expected, actual)

    def test_not_in_group(self):
        class AuthStub():
            def verify(self, credentials):
                return True

            def ingroup(self, username, groupname):
                return False

        self._writeJson(
            "hello.json", {
                ".special:restricted": {
                    "users": ["Lucy"],
                    "groups": ["Backup operators"]
                },
                "world": "Hello"
            }
        )

        finder = self._finder
        finder.auth_provider = AuthStub()
        finder.access_as(trust.Credentials("John", "demo"))
        with self.assertRaises(PermissionError) as context:
            finder.find("/hello/world")

    def test_special_value(self):
        class AuthStub():
            def verify(self, credentials):
                return True

        self._writeJson(
            "hello.json", {
                ".special:restricted": {
                    "users": ["John"]
                },
                ".special:value": "Hello"
            }
        )

        finder = self._finder
        finder.auth_provider = AuthStub()
        finder.access_as(trust.Credentials("John", "demo"))
        actual = finder.find("/hello")
        expected = "Hello"

        self.assertEqual(expected, actual)

    def test_special_value_object(self):
        class AuthStub():
            def verify(self, credentials):
                return True

        self._writeJson(
            "hello.json", {
                ".special:restricted": {
                    "users": ["John"]
                },
                ".special:value": {
                    "world": "Hello"
                }
            }
        )

        finder = self._finder
        finder.auth_provider = AuthStub()
        finder.access_as(trust.Credentials("John", "demo"))
        actual = finder.find("/hello/world")
        expected = "Hello"

        self.assertEqual(expected, actual)

    def test_no_auth_if_public(self):
        class AuthMock():
            def __init__(self):
                self.verify_called = False

            def verify(self, credentials):
                self.verify_called = True
                return True

        self._writeJson(
            "hello.json", {
                "world": "Hello, World"
            }
        )

        mock = AuthMock()
        finder = self._finder
        finder.auth_provider = mock
        finder.access_as(trust.Credentials("John", "demo"))
        actual = finder.find("/hello/world")

        self.assertFalse(mock.verify_called)



import trust
import json


class JsonFormatter():
    def __init__(self, finder):
        self._finder = finder

    def process(self, query, credentials, optional=False):
        self._finder.access_as(credentials)

        try:
            return json.dumps({
                "result": self._finder.find(query, optional=optional)
            })
        except trust.exceptions.AuthenticationException:
            return self._generate_error_response([
                self._generate_error_object(
                    "credentials-invalid",
                    "Either the user name doesn't exist or the password " +
                    "is invalid."
                )
            ])
        except PermissionError:
            return self._generate_error_response([
                self._generate_error_object(
                    "permission-required",
                    "A permission is required to be able to access the " +
                    "requested element."
                )
            ])
        except trust.exceptions.NodeNotFoundException:
            return self._generate_error_response([
                self._generate_error_object(
                    "node-not-found",
                    "The specified node doesn't exist."
                )
            ])
        except trust.exceptions.CircularInheritanceException:
            return self._generate_error_response([
                self._generate_error_object(
                    "circular-inheritance",
                    "The data contains a circular inheritance."
                )
            ])

    def reject_invalid(self):
        return self._generate_error_response([
            self._generate_error_object(
                "query-invalid",
                "The query contains invalid characters or parts."
            )
        ])

    def _generate_error_object(self, error_type, description):
        return {
            "type": error_type,
            "uri": "http://services.pelicandd.com/trust/error/" + error_type,
            "description": description
        }

    def _generate_error_response(self, error_objects):
        return json.dumps({
            "errors": error_objects
        })



import trust
from .common_tests import CommonTests


class TestFileFinder(CommonTests):
    def test_empty_path(self):
        with self.assertRaises(ValueError) as context:
            self._finder.find("")

    def test_invalid_path(self):
        with self.assertRaises(ValueError) as context:
            self._finder.find("hello")

    def test_missing_file(self):
        with self.assertRaises(trust.NodeNotFoundException) as context:
            self._finder.find("/hello")

    def test_missing_node(self):
        self._writeJson(
            "hello.json", {
                "world": "Hello, World!"
            }
        )

        with self.assertRaises(trust.NodeNotFoundException) as context:
            self._finder.find("/hello/abc")

    def test_missing_file_optional(self):
        actual = self._finder.find("/hello", optional=True)
        self.assertIsNone(actual)

    def test_missing_node_optional(self):
        self._writeJson(
            "hello.json", {
                "world": "Hello, World!"
            }
        )

        actual = self._finder.find("/hello/abc", optional=True)
        self.assertIsNone(actual)

    def test_root_value(self):
        self._writeJson(
            "hello.json", {
                "world": "Hello, World!"
            }
        )

        actual = self._finder.find("/hello/world")
        expected = "Hello, World!"
        self.assertEqual(expected, actual)

    def test_child_value(self):
        self._writeJson(
            "hello.json", {
                "world": {
                    "child": "Hello, World!"
                }
            }
        )

        actual = self._finder.find("/hello/world/child")
        expected = "Hello, World!"
        self.assertEqual(expected, actual)

    def test_value_in_directory(self):
        self._writeJson(
            "hello/child.json", {
                "world": "Hello, World!"
            }
        )

        actual = self._finder.find("/hello/child/world")
        expected = "Hello, World!"
        self.assertEqual(expected, actual)

    def test_int(self):
        self._writeJson(
            "hello.json", {
                "world": 3
            }
        )

        actual = self._finder.find("/hello/world")
        expected = 3
        self.assertEqual(expected, actual)

    def test_float(self):
        self._writeJson(
            "hello.json", {
                "world": 3.14
            }
        )

        actual = self._finder.find("/hello/world")
        expected = 3.14
        self.assertEqual(expected, actual)

    def test_array(self):
        self._writeJson(
            "hello.json", {
                "world": [2, 5, 1, 4]
            }
        )

        actual = self._finder.find("/hello/world")
        expected = [2, 5, 1, 4]
        self.assertEqual(expected, actual)

    def test_indexes(self):
        self._writeJson(
            "hello.json", {
                "world": [2, 5, 1, 4]
            }
        )

        actual = self._finder.find("/hello/world/1")
        expected = 5
        self.assertEqual(expected, actual)

    def test_keys(self):
        self._writeJson(
            "hello.json", {
                "world": {
                    "a": 4,
                    "f": 1,
                    "c": 8
                }
            }
        )

        actual = self._finder.find("/hello/world/.keys")
        expected = ["a", "c", "f"]
        self.assertEqual(expected, actual)

    def test_keys_escape(self):
        self._writeJson(
            "hello.json", {
                "world": {
                    ".keys": "Hello, World!"
                }
            }
        )

        actual = self._finder.find("/hello/world/.plain:.keys")
        expected = "Hello, World!"
        self.assertEqual(expected, actual)

    def test_double_escape(self):
        self._writeJson(
            "hello.json", {
                ".plain:.plain:.keys": "Hello, World!"
            }
        )

        actual = self._finder.find("/hello/.plain:.plain:.plain:.keys")
        expected = "Hello, World!"
        self.assertEqual(expected, actual)

    def test_keys_escape_go_inside(self):
        self._writeJson(
            "hello.json", {
                "world": {
                    ".keys": {
                        "a": "Hello, World!"
                    }
                }
            }
        )

        actual = self._finder.find("/hello/world/.plain:.keys/a")
        expected = "Hello, World!"
        self.assertEqual(expected, actual)

    def test_keys_above_file(self):
        self._writeJson("hello.json", {"a": 2})
        self._writeJson("world.json", {"a": 7})
        actual = self._finder.find("/.keys")
        expected = ["hello", "world"]
        self.assertEqual(expected, actual)

    def test_keys_above_file_deep(self):
        self._writeJson("parent/hello.json", {"a": 2})
        self._writeJson("parent/world.json", {"a": 7})
        actual = self._finder.find("/parent/.keys")
        expected = ["hello", "world"]
        self.assertEqual(expected, actual)

    def test_json(self):
        self._writeJson(
            "hello.json", {
                "world": {
                    "a": 5
                }
            }
        )

        actual = self._finder.find("/hello/world")
        expected = {"a": 5}
        self.assertEqual(expected, actual)

    def test_root(self):
        self._writeJson(
            "hello.json", {
                "world": 5
            }
        )

        actual = self._finder.find("/hello")
        expected = {"world": 5}
        self.assertEqual(expected, actual)

    def test_inheritance(self):
        self._writeJson(
            "parent.json", {
                "world": 5
            }
        )

        self._writeJson(
            "child.json", {
                ".special:inherit": "/parent",
                "hello": 2
            }
        )

        actual = self._finder.find("/child/world")
        expected = 5
        self.assertEqual(expected, actual)

    def test_deep_inheritance(self):
        self._writeJson(
            "parent.json", {
                "world": 5
            }
        )

        self._writeJson(
            "child.json", {
                "sub": {
                    ".special:inherit": "/parent",
                    "hello": 2
                }
            }
        )

        actual = self._finder.find("/child/sub/world")
        expected = 5
        self.assertEqual(expected, actual)

    def test_inheritance_inside(self):
        self._writeJson(
            "parent.json", {
                "world": 5
            }
        )

        self._writeJson(
            "child.json", {
                "sub": {
                    ".special:inherit": "/parent",
                    "hello": 2
                }
            }
        )

        actual = self._finder.find("/child")
        expected = {"sub": {"hello": 2, "world": 5}}
        self.assertEqual(expected, actual)

    def test_inheritance_replacing(self):
        self._writeJson(
            "parent.json", {
                "world": 5
            }
        )

        self._writeJson(
            "child.json", {
                ".special:inherit": "/parent",
                "hello": 2,
                "world": 4
            }
        )

        actual = self._finder.find("/child/world")
        expected = 4
        self.assertEqual(expected, actual)

    def test_inheritance_replacing_by_value(self):
        self._writeJson(
            "parent.json", {
                "hello": 4
            }
        )

        self._writeJson(
            "child.json", {
                "hello": {
                    ".special:inherit": "/parent/hello",
                }
            }
        )

        actual = self._finder.find("/child")
        expected = {"hello": 4}
        self.assertEqual(expected, actual)

    def test_inheritance_replacing_by_value_direct(self):
        self._writeJson(
            "parent.json", {
                "hello": 4
            }
        )

        self._writeJson(
            "child.json", {
                "hello": {
                    ".special:inherit": "/parent/hello",
                }
            }
        )

        actual = self._finder.find("/child/hello")
        expected = 4
        self.assertEqual(expected, actual)

    def test_deep_inheritance_replacing(self):
        self._writeJson(
            "parent.json", {
                "world": {
                    "a": 3,
                    "b": 6
                }
            }
        )

        self._writeJson(
            "child.json", {
                ".special:inherit": "/parent",
                "world": {
                    ".special:actions": ["replace"],
                    "b": 4,
                    "c": 8
                }
            }
        )

        actual = self._finder.find("/child/world")
        expected = {"b": 4, "c": 8}
        self.assertEqual(expected, actual)

    def test_deep_inheritance_merge(self):
        self._writeJson(
            "parent.json", {
                "world": {
                    "a": 3,
                    "b": 6
                }
            }
        )

        self._writeJson(
            "child.json", {
                ".special:inherit": "/parent",
                "world": {
                    ".special:actions": ["merge"],
                    "b": 4,
                    "c": 8
                }
            }
        )

        actual = self._finder.find("/child/world")
        expected = {"a": 3, "b": 4, "c": 8}
        self.assertEqual(expected, actual)

    def test_inheritance_array_replace(self):
        self._writeJson(
            "parent.json", {
                "world": [3, 6]
            }
        )

        self._writeJson(
            "child.json", {
                ".special:inherit": "/parent",
                "world": [2, 1]
            }
        )

        actual = sorted(self._finder.find("/child/world"))
        expected = [1, 2]
        self.assertSequenceEqual(expected, actual)

    def test_inheritance_array_add(self):
        self._writeJson(
            "parent.json", {
                "world": [3, 6]
            }
        )

        self._writeJson(
            "child.json", {
                ".special:inherit": "/parent",
                "world": {
                    ".special:actions": ["add"],
                    ".special:values": [2, 1, 3]
                }
            }
        )

        actual = sorted(self._finder.find("/child/world"))
        expected = [1, 2, 3, 3, 6]
        self.assertSequenceEqual(expected, actual)

    def test_inheritance_array_merge(self):
        self._writeJson(
            "parent.json", {
                "world": [3, 6]
            }
        )

        self._writeJson(
            "child.json", {
                ".special:inherit": "/parent",
                "world": {
                    ".special:actions": ["merge"],
                    ".special:values": [2, 1, 3]
                }
            }
        )

        actual = sorted(self._finder.find("/child/world"))
        expected = [1, 2, 3, 6]
        self.assertSequenceEqual(expected, actual)

    def test_inheritance_array_add_duplicates(self):
        self._writeJson(
            "parent.json", {
                "world": [3, 6, 6]
            }
        )

        self._writeJson(
            "child.json", {
                ".special:inherit": "/parent",
                "world": {
                    ".special:actions": ["add"],
                    ".special:values": [2, 1, 1, 3]
                }
            }
        )

        actual = sorted(self._finder.find("/child/world"))
        expected = [1, 1, 2, 3, 3, 6, 6]
        self.assertSequenceEqual(expected, actual)

    def test_inheritance_array_merge_duplicates(self):
        self._writeJson(
            "parent.json", {
                "world": [3, 6, 6]
            }
        )

        self._writeJson(
            "child.json", {
                ".special:inherit": "/parent",
                "world": {
                    ".special:actions": ["merge"],
                    ".special:values": [2, 1, 1, 3]
                }
            }
        )

        actual = sorted(self._finder.find("/child/world"))
        expected = [1, 2, 3, 6]
        self.assertSequenceEqual(expected, actual)

    def test_replacing_list_with_object(self):
        self._writeJson(
            "parent.json", {
                "world": [3, 2, 7]
            }
        )

        self._writeJson(
            "child.json", {
                ".special:inherit": "/parent",
                "world": {
                    "a": "Hello",
                    "b": "World"
                }
            }
        )

        actual = self._finder.find("/child/world")
        expected = [3, 2, 7]
        self.assertEqual(expected, actual)

    def test_replacing_object_with_list(self):
        self._writeJson(
            "parent.json", {
                "world": {
                    "a": "Hello",
                    "b": "World"
                }
            }
        )

        self._writeJson(
            "child.json", {
                ".special:inherit": "/parent",
                "world": [3, 2, 7]
            }
        )

        actual = self._finder.find("/child/world")
        expected = [3, 2, 7]
        self.assertSequenceEqual(expected, actual)

    def test_whole_file(self):
        self._writeJson(
            "demo/hello.json", {
                "world": "Hello, World!"
            }
        )

        actual = self._finder.find("/demo/hello")
        expected = {
            "world": "Hello, World!"
        }

        self.assertEqual(expected, actual)

    def test_whole_directory(self):
        self._writeJson(
            "demo/hello.json", {
                "world": "Hello, World!"
            }
        )

        actual = self._finder.find("/demo")
        expected = {
            "hello": {
                "world": "Hello, World!"
            }
        }

        self.assertEqual(expected, actual)

    def test_whole_directory_hierarchy(self):
        self._writeJson(
            "demo/example/hello.json", {
                "world": "Hello, World!"
            }
        )

        actual = self._finder.find("/demo")
        expected = {
            "example": {
                "hello": {
                    "world": "Hello, World!"
                }
            }
        }

        self.assertEqual(expected, actual)



import trust
import json


class CompleteFormatter():
    def __init__(self, finder):
        self._finder = finder

    def process(self, query, credentials, optional=False):
        self._finder.access_as(credentials)
        result = self._finder.find(query, optional=optional)
        return json.dumps(result, sort_keys=True)

    def reject_invalid(self):
        raise trust.InvalidQueryException



from abc import ABCMeta, abstractmethod
import trust
from .logged_tests import LoggedTests


class FormatterTests(LoggedTests, metaclass=ABCMeta):
    @abstractmethod
    def _get_formatter(self, finder):
        pass

    def _process_default(self, value):
        formatter = self._get_formatter(FinderStubWithValue(value))
        return formatter.process("/", trust.Credentials.get_empty())


class FinderStubWithValue():
    def __init__(self, value):
        self._value = value

    def access_as(self, credentials):
        pass

    def find(self, query, optional):
        return self._value



import trust
import json
from trust.exceptions import AuthenticationException, FormatterException
from .logged_tests import LoggedTests


class TestTrust(LoggedTests):
    def test_response_mode_json(self):
        actual = self._process_list_using_response_type("json")
        expected = '{"result": [3, 7, 1]}'
        self.assertEqual(expected, actual)

    def test_response_mode_complete(self):
        actual = self._process_list_using_response_type("complete")
        expected = "[3, 7, 1]"
        self.assertEqual(expected, actual)

    def test_response_mode_text(self):
        actual = self._process_list_using_response_type("text")
        expected = "3\n7\n1"
        self.assertEqual(expected, actual)

    def test_response_mode_empty(self):
        actual = self._process_list_using_response_type("")
        expected = "[3, 7, 1]"
        self.assertEqual(expected, actual)

    def test_response_mode_other(self):
        with self.assertRaises(FormatterException) as context:
            actual = self._process_list_using_response_type("something else")

    def test_no_credentials(self):
        class FinderMock():
            def __init__(self):
                self.find_called = False

            def access_as(self, credentials):
                pass

            def find(self, query, optional):
                self.find_called = True

        mock = FinderMock()
        result = trust.Trust(mock).process(
            "/", "complete", trust.Credentials.get_empty()
        )

        self.assertTrue(mock.find_called)

    def _process_list_using_response_type(self, response_type):
        class FinderStub():
            def access_as(self, credentials):
                pass

            def find(self, query, optional):
                return [3, 7, 1]

        return trust.Trust(FinderStub()).process(
            "/", response_type, trust.Credentials.get_empty()
        )


class FinderStubAlwaysFailAuthentication():
    def access_as(self, credentials):
        pass

    def find(self, query, optional):
        raise AuthenticationException



from .exceptions import AuthenticationException, FormatterException, \
    InvalidQueryException, NodeNotFoundException
from .file_finder import FileFinder
from .trust import Trust, Credentials
from .trust_blueprint import trust_blueprint



import trust
import trust.auth
from .common_tests import CommonTests


class TestInDataAuthProvider(CommonTests):
    def test_verify_right(self):
        self._writeJson(
            "_users.json", {
                "sample": {
                    "hash": ("$pbkdf2-sha256$10$1hpjzFnLubf2/j9nrPVeaw$DJBUI" +
                             "hmaa2U8.11dfh5cueJBAEfK0MWflMNL7oCJXpY")
                }
            })

        result = self._auth.verify(
            trust.Credentials("sample", "correct password")
        )

        self.assertTrue(result)

    def test_verify_wrong(self):
        self._writeJson(
            "_users.json", {
                "sample": {
                    "hash": ("$pbkdf2-sha256$10$1hpjzFnLubf2/j9nrPVeaw$DJBUI" +
                             "hmaa2U8.11dfh5cueJBAEfK0MWflMNL7oCJXpY")
                }
            })

        result = self._auth.verify(
            trust.Credentials("sample", "I'm the wrong password")
        )

        self.assertFalse(result)

    def test_ingroup_right(self):
        self._writeJson(
            "_users.json", {
                "sample": {
                    "member-of": ["a"]
                }
            })

        self._writeJson(
            "_groups.json", {
                "a": {}
            }
        )

        result = self._auth.ingroup("sample", "a")
        self.assertTrue(result)

    def test_ingroup_wrong(self):
        self._writeJson(
            "_users.json", {
                "sample": {
                    "member-of": ["a"]
                }
            })

        self._writeJson(
            "_groups.json", {
                "a": {}
            }
        )

        result = self._auth.ingroup("sample", "b")
        self.assertFalse(result)

    def test_ingroup_tree(self):
        self._writeJson(
            "_users.json", {
                "sample": {
                    "member-of": ["a"]
                }
            })

        self._writeJson(
            "_groups.json", {
                "a": {
                    "member-of": ["b"]
                }
            }
        )

        result = self._auth.ingroup("sample", "b")
        self.assertTrue(result)

    def test_ingroup_circular(self):
        self._writeJson(
            "_users.json", {
                "sample": {
                    "member-of": ["a"]
                }
            })

        self._writeJson(
            "_groups.json", {
                "a": {
                    "member-of": ["b"]
                },
                "b": {
                    "member-of": ["a"]
                }
            }
        )

        result = self._auth.ingroup("sample", "c")
        self.assertFalse(result)

    def test_ingroup_self_referencing(self):
        self._writeJson(
            "_users.json", {
                "sample": {
                    "member-of": ["a"]
                }
            })

        self._writeJson(
            "_groups.json", {
                "a": {
                    "member-of": ["a"]
                }
            }
        )

        result = self._auth.ingroup("sample", "b")
        self.assertFalse(result)

    @property
    def _auth(self):
        return trust.auth.InDataAuthProvider(self._finder, self._logger)



import trust
import json


class TextFormatter():
    def __init__(self, finder):
        self._finder = finder

    def process(self, query, credentials, optional=False):
        self._finder.access_as(credentials)
        result = self._finder.find(query, optional=optional)

        if result is None:
            return ""
        if type(result) is list:
            return "\n".join(
                [self._process_list_element(c) for c in result]
            )
        elif type(result) is str:
            return result

        return json.dumps(result, sort_keys=True)

    def reject_invalid(self):
        raise trust.InvalidQueryException

    def _process_list_element(self, element):
        return (
            str(
                json.dumps(element, sort_keys=True)
                if type(element) is dict
                else element
            )
        )



class NodeNotFoundException(Exception):
    pass


class AuthenticationException(Exception):
    pass


class FormatterException(Exception):
    pass


class InvalidQueryException(Exception):
    pass


class CircularInheritanceException(Exception):
    pass



import trust
import trust.audit
import json
import logging
import os
from .auth import InDataAuthProvider
from .exceptions import AuthenticationException, NodeNotFoundException, \
        CircularInheritanceException
from .zipper import Zipper


class FileFinder():
    def __init__(self, data_path, logger=None, audit=None):
        self._data_path = data_path
        self._logger = logger or logging.getLogger(__name__)
        self._audit = audit or trust.audit.NullAudit()
        self._is_authenticated = False
        self._credentials = None
        self._auth_provider = None

    @property
    def username(self):
        return (None
                if self._credentials is None
                else self._credentials.username)

    @property
    def auth_provider(self):
        return self._auth_provider or InDataAuthProvider(self, self._logger)

    @auth_provider.setter
    def auth_provider(self, value):
        self._auth_provider = value

    def find(self, path, optional=False, previous_inheritance_paths=[]):
        if len(path) == 0:
            raise ValueError("The path shouldn't be empty.")

        if path[0] != "/":
            raise ValueError(
                "The path should start with a slash. The character '%s' is "
                "unexpected." % (path[0],))

        self._logger.info("The loader is finding %s.", path)

        result = self._find(path, optional, previous_inheritance_paths)

        if path == "/":
            del result["_users"]
            del result["_groups"]

        return result

    def _find(self, path, optional=False, previous_inheritance_paths=[]):
        parts = self._split_path(path)
        json, remaining_parts = self._find_json(parts)
        parts_taken = parts[:len(parts) - len(remaining_parts)]

        if json is None:
            if parts_taken[-1] == ".keys":
                names = sorted(self._list_files_in_directory(parts_taken))
                return list(names)

            directory = self._explore_directory(parts_taken + remaining_parts)
            if not optional and directory is None:
                raise NodeNotFoundException

            return directory

        return self._extract(
            parts_taken, remaining_parts, json, optional,
            previous_inheritance_paths=previous_inheritance_paths
        )

    def access_as(self, credentials):
        self._is_authenticated = False
        self._credentials = credentials

    def _authenticate(self):
        if self.username is None:
            self._logger.debug("No user to authenticate.")
            return

        self._logger.debug(
            "Authenticating user %s.", self.username
        )

        auth_result = self.auth_provider.verify(self._credentials)
        self._logger.info("Audit: %s." % (self._audit,))
        self._logger.info("Authentication result: %s." % (auth_result,))
        self._audit.user_verified(self.username, auth_result)

        if not auth_result:
            self._logger.info("User %s failed to authenticate.", self.username)
            raise AuthenticationException

        self._logger.info("User %s logged on.", self.username)

    def _extract(self, path, parts, obj, optional, skip_restricted=False,
                 previous_inheritance_paths=[]):
        self._logger.debug("Processing %s: %s.", path, parts)

        keys = obj.keys() if type(obj) is dict else []

        if ".special:restricted" in keys:
            if not self._verify_access(obj[".special:restricted"]):
                if skip_restricted:
                    return None

                self._audit.restricted_node_accessed(
                    self.username, "/".join(path), False
                )
                raise PermissionError()

            self._audit.restricted_node_accessed(
                self.username, "/".join(path), True
            )

            del obj[".special:restricted"]

            if ".special:value" in keys:
                return self._extract(
                    path, parts, obj[".special:value"], optional,
                    previous_inheritance_paths=previous_inheritance_paths
                )

        if ".special:inherit" in keys:
            obj = self._inherit(obj, previous_inheritance_paths)

        if len(parts) == 0:
            if type(obj) is list:
                self._logger.debug("The object is an array.")
                result = []
                for current in obj:
                    array_item = self._extract(
                        path, [], current, optional,
                        previous_inheritance_paths=previous_inheritance_paths
                    )

                    result.append(array_item)

                return result

            if type(obj) is not dict:
                self._logger.debug("The object is a scalar.")
                return obj

            result = {}
            for c in obj:
                result[c] = self._extract(
                    path, [], obj[c], optional, skip_restricted=True,
                    previous_inheritance_paths=previous_inheritance_paths
                )

            return result

        name = parts[0]

        special = self._process_special_keys(name, obj)
        if special is not None:
            return special

        if name.startswith(".plain:"):
            name = name[7:]

        try:
            current = obj[name]
        except TypeError:
            current = obj[int(name)]
        except KeyError:
            if optional:
                return None

            # Breaking the stack is intentional here.
            raise NodeNotFoundException

        if len(parts) == 1:
            if type(current) is not dict and type(current) is not list:
                return current

            return self._extract(
                path + parts, [], current, optional,
                previous_inheritance_paths=previous_inheritance_paths
            )

        return self._extract(
            path + [name], parts[1:], current, optional,
            previous_inheritance_paths=previous_inheritance_paths
        )

    def _verify_access(self, restrictions):
        if not self._is_authenticated:
            self._authenticate()

        if self.username is None:
            self._logger.debug("A guest is trying to access restricted data.")
            return False

        users = restrictions.get("users", [])
        self._logger.debug("The following users can access data: %s.", users)
        if self.username in users:
            return True

        groups = restrictions.get("groups", [])
        self._logger.debug("The following groups can access data: %s.", groups)
        for group in groups:
            if self.auth_provider.ingroup(self.username, group):
                return True

        return False

    def _explore_directory(self, parts):
        """
        Generates the JSON representation of the directory and JSON files
        within the directory and the subdirectories.

        Returns either an object corresponding to the JSON representation of
        the data, or None if the directory doesn't exist.
        """
        dir_path = os.path.join(self._data_path, "/".join(parts))
        self._logger.debug("Exploring the directory %s (%s).", parts, dir_path)
        base_path = "".join(["/" + c for c in parts])
        response = {}

        if not os.path.isdir(dir_path):
            return None

        for name in os.listdir(dir_path):
            path = os.path.join(dir_path, name)
            if path.endswith(".json") and os.path.isfile(path):
                short_name = name[:-5]
                query_path = "/".join([base_path, short_name])
                response[short_name] = self.find(query_path)

            if os.path.isdir(path):
                response[name] = self._explore_directory(parts + [name])

        return response

    def _list_files_in_directory(self, parts):
        dir_path = os.path.join(self._data_path, "/".join(parts[:-1]))
        self._logger.debug("Listing contents of directory %s.", dir_path)
        for name in os.listdir(dir_path):
            path = os.path.join(dir_path, name)
            if path.endswith(".json") and os.path.isfile(path):
                short_name = name[:-5]
                yield short_name

    def _process_special_keys(self, name, obj):
        if name == '.keys':
            keys = obj.keys()
            return sorted(obj.keys())

        return None

    def _inherit(self, obj, previous_inheritance_paths):
        parent_path = obj[".special:inherit"]
        if parent_path in previous_inheritance_paths:
            raise CircularInheritanceException()

        self._logger.debug("Inheriting on %s.", parent_path)
        parent = self.find(
            parent_path, False, previous_inheritance_paths + [parent_path]
        )

        self._logger.debug("Got %s through inheritance. Combining.", parent)
        del obj[".special:inherit"]
        return Zipper(obj, parent, self._logger).zip()

    def _split_path(self, path):
        normalized_path = path.lstrip("/")
        return normalized_path.split("/")

    def _find_json(self, parts):
        """
        Finds the first JSON file which matches the parts.

        Returns a tuple containing:

         - The JSON corresponding to the first JSON file matching the criteria,
           or None if there are no matches.
         - The parts which were not used for the path to the JSON file.
        """
        remaining_parts = parts
        for file_path in self._list_json_file_candidates(parts):
            remaining_parts = remaining_parts[1:]
            full_path = os.path.join(self._data_path, file_path)
            self._logger.debug("Checking for %s.", full_path)
            if os.path.isfile(full_path):
                with open(full_path) as f:
                    return (json.load(f), remaining_parts)

        return (None, remaining_parts)

    def _list_json_file_candidates(self, parts):
        """
        Lists the JSON file candidates which may be used based on the current
        parts list. For instance, the parts ["a", "b", "c"] will yield three
        paths:
         - a.json
         - a/b.json
         - a/b/c.json
        """
        for path in self._aggregate_parts_in_path(parts):
            yield path[:-1] + ".json"

    def _aggregate_parts_in_path(self, parts):
        """
        Aggregates the path parts to progressively form longer and longer
        paths. For instance, the parts ["a", "b", "c"] will yield three paths:
         - a/
         - a/b/
         - a/b/c/
        The paths contain trailing slash but no leading slash.
        """
        path = ""
        for part in parts:
            path = path + part + "/"
            yield path



from .audit_to_log import AuditToLog
import logging
import logging.handlers


class AuditToSyslog(AuditToLog):
    def __init__(self,
                 name,
                 address="/dev/log",
                 facility=logging.handlers.SysLogHandler.LOG_AUTH):
        logHandler = logging.handlers.SysLogHandler(
            address=address, facility=facility
        )

        formatter = logging.Formatter(name + ": [%(levelname)s] %(message)s")
        logHandler.setFormatter(formatter)

        logHandler.setLevel(logging.INFO)

        log = logging.getLogger(name)
        log.addHandler(logHandler)
        log.setLevel(logging.INFO)

        super(AuditToSyslog, self).__init__(log)



#!/usr/bin/env python

import setuptools

setuptools.setup(
    name="trust",
    version="1.0.8",
    description="A hierarchical database engine with read-only access to "
                "data, ideally used for cross-application settings and "
                "infrastructure description.",
    long_description=open("README.markdown").read(),
    author="Arseni Mourzenko",
    author_email="arseni.mourzenko@pelicandd.com",
    url="http://go.pelicandd.com/n/python-trust",
    license="MIT",
    keywords="database json settings",
    install_requires=[
        "flask==0.10.1",
        "markdown==2.6.1",
        "passlib==1.6.2"
    ],
    packages=setuptools.find_packages(
        exclude=["tests", "tests.*", "www", "www.*"]
    ),
    # Since it seems that there is no way to include data with `data_files`,
    # this looks like the only way to include the deployment script and other
    # static files. Since the name of the package is mandatory, any name which
    # exists in the project will work.
    package_data={
        "trust": [
            "../localtrust.py",
            "../extras/*"
        ]
    }
)



class AuditToLog:
    def __init__(self, log_handler):
        self._log = log_handler

    def user_verified(self, user_name, is_success):
        if is_success:
            self._log.info("Access granted for user %s." % (user_name,))
        else:
            self._log.warn("Access denied for user %s." % (user_name,))

    def restricted_node_accessed(self, user_name, path, is_granted):
        if is_granted:
            self._log.info(
                "The user %s is authorized to access %s." % (user_name, path)
            )
        else:
            self._log.warn(
                "The user %s is denied to access %s." % (user_name, path)
            )



import json
import os
import tempfile
import shutil
import trust
from abc import ABCMeta
from .logged_tests import LoggedTests


class CommonTests(LoggedTests, metaclass=ABCMeta):
    def setUp(self):
        super().setUp()
        self._temp_path = tempfile.mkdtemp(prefix="createvm-test-trust")

    def tearDown(self):
        shutil.rmtree(self._temp_path)

    @property
    def _finder(self):
        return trust.FileFinder(data_path=self._temp_path)

    def _writeJson(self, path, obj):
        full_path = os.path.join(self._temp_path, path)
        os.makedirs(os.path.dirname(full_path), exist_ok=True)
        with open(full_path, "w") as f:
            json.dump(obj, f)
            self._logger.debug("Wrote the JSON file %s.", full_path)



import collections
import flask
import json
import markdown
import os
import requests


docs_blueprint = flask.Blueprint(
    "www/docs", __name__,
    template_folder="templates",
    static_folder="static"
)


@docs_blueprint.route("/")
def index():
    app_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    global_file = "tests/system/data/data-tests.json"

    with open(os.path.join(app_path, global_file)) as f:
        data = json.load(f)

    illustrations = sorted(
        [c for c
         in data["system-tests"].items()
         if "show-on-site" in c[1]],
        key=lambda t: t[1]["position"]
    )

    with open(os.path.join(app_path, "README.markdown")) as f:
        docs_markdown = f.read()

    return flask.render_template(
        "global.html",
        illustrations=illustrations,
        request=flask.request,
        base_uri=flask.request.url_root[7:],
        documentation=markdown.markdown(docs_markdown)
    )


@docs_blueprint.route("/css")
def css():
    return docs_blueprint.send_static_file("g.min.css")


@docs_blueprint.route("/logo")
def logo():
    return docs_blueprint.send_static_file("trust-logo-top.png")


@docs_blueprint.route("/logo-bottom")
def logo_bottom():
    return docs_blueprint.send_static_file("trust-logo-bottom.png")


@docs_blueprint.route("/proxy", methods=["GET"])
def query_proxy():
    app_url = flask.request.host_url
    query = flask.request.args.get("query", "")
    response = requests.get(
        app_url + "service/" + query + "?response-mode=json&optional=1",
        auth=("William", "demo")
    )

    return (response.text, response.status_code)



#!/usr/bin/env python

import argparse
from passlib.hash import pbkdf2_sha256


rounds_map = {
    "poor": 10,
    "normal": 50000,
    "strong": 100000,
}

parser = argparse.ArgumentParser(
    description="Generates a PBKDF2 password hash."
)

parser.add_argument(
    "--level",
    choices=rounds_map.keys(),
    default="strong",
    help="The level of encryption."
)

parser.add_argument("password", help="The password to hash.")

args = parser.parse_args()

result = pbkdf2_sha256.encrypt(
    args.password,
    rounds=rounds_map[args.level],
    salt_size=16
)

print(result)
